%% Section 1, understanding the model

% 1.1 initialize Cobra toolbox

initCobraToolbox

% 1.2 load the E. coli metabolic reconstruction

% if you have Matlab 201uncomment following line

model = readCbModel('iJO1366.xml')

% if you have Matlab 2019 uncomment and execute the following two lines

model = load('model_fix.mat')
model = model.model

% 1.3 print the list of reactions and of metabolites (a long list)

model.rxns
model.mets

% 1.4 build a table with reaction codes and the description of the reactions

table(model.rxns, model.rxnNames)

% 1.5 build a table with metabolite codes and their names

table(model.mets, model.metNames)

% 1.6 Biomass assembly reaction for this model is
% "BIOMASS_Ec_iJO1366_WT_53p95M"
% Set this reaction as the objective function:

model = changeObjective(model, 'BIOMASS_Ec_iJO1366_WT_53p95M')

% 1.7 solve the FBA problem for the current selected objective function and
% make a bar plot of the resulting fluxes (only those different (~=) from
% 0)

sol = optimizeCbModel(model)
figure
bar(sol.x(sol.x~=0))

% 1.8 identify exchange reactions

Exchange_reactions = model.rxns(findExcRxns(model))

% 1.9 exclude demand reactions (those starting with "DM_") and keep only true exchange
% reactions (those starting with "EX_")

EX_reactions = Exchange_reactions(strmatch('EX_', Exchange_reactions))

% 1.10 get EX reactions lower bounds

EX_reactions_lb = model.lb(findRxnIDs(model, EX_reactions))

% 1.11 make a table with the output

table (EX_reactions, EX_reactions_lb)

% 1.12 select the lower bounds that are < 0 and make a table with the output

lb_lower_0 = EX_reactions_lb<0

table(EX_reactions(lb_lower_0),  EX_reactions_lb(lb_lower_0))

% 1.13 add EX reactions description to the table

    % get EX reactions description

        %find EX reaction index in the model
        findRxnIDs(model, EX_reactions)

    EX_reactions_description = model.rxnNames(findRxnIDs(model, EX_reactions))

table(EX_reactions(lb_lower_0),  EX_reactions_lb(lb_lower_0), EX_reactions_description(lb_lower_0))


%% Section 2, playing with growth conditions

% 2.1 change glucose uptake rate and check the effect on growth

    % set glucose uptake to a lower value
    model_low_glc = changeRxnBounds(model, 'EX_glc__D(e)', -3.5, 'l');
    sol_low_glc = optimizeCbModel(model_low_glc)

    %set glucose uptake to 0
    model_no_glc = changeRxnBounds(model, 'EX_glc__D(e)', 0, 'l');
    sol_no_glc = optimizeCbModel(model_no_glc)

% 2.2 change carbon source to glutamate (exchange reaction for xylose is EX_glu__D_e)

model_glu = changeRxnBounds(model_no_glc, 'EX_glu__L(e)', -10, 'l');
sol_glu = optimizeCbModel(model_glu)

% 2.3 test anaerobic growth on glucose

model_anaerobic = changeRxnBounds(model, 'EX_o2(e)', 0, 'l');
sol_anaerobic = optimizeCbModel(model_anaerobic)
    


%% Section 3, playing with E. coli genes


% 3.1 perform a genome-wide gene essentiality analysis using glucose as the
% single carbon source

[grRatio_glc,grRateKO,grRateWT,delRxns,hasEffect] = singleGeneDeletion(model);

% print a table with the results

table(model.genes, grRatio_glc)

% plot the results

plot( (1:length(grRatio_glc)),sort(grRatio_glc), 'o', 'color', 'blue')
% 3.2 perform a genome-wide gene essentiality analysis using glutamate as the
% single carbon source

[grRatio_glu,grRateKO,grRateWT,delRxns,hasEffect] = singleGeneDeletion(model_glu);

% print a table with the results

table(model_glu.genes, grRatio_glu)

% plot the differences between the two essential genes datasets

figure 
plot( (1:length(grRatio_glc)),sort(grRatio_glc), 'o', 'color', 'blue')
hold on
plot( (1:length(grRatio_glu)),sort(grRatio_glu), 'o', 'color', 'red')

